/* GnomeCard - a graphical contact manager.
 *
 * gnomecard.c: This file is part of GnomeCard.
 * $Id: gnomecard.c,v 1.80.2.20 2002/07/20 20:04:17 srittau Exp $
 * 
 * Copyright (C) 1999 The Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changed in 2002 by Sebastian Rittau <srittau@jroger.in-berlin.de>
 * for the GNOME 2 port.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <time.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>

#include <locale.h>

#include <glib.h>
#include <gnome.h>
#include <orbit/orbit.h>

#include <mimedir/mimedir-vcard.h>
#include <libversit/vcc.h>

#include "canvas.h"
#include "card.h"
#include "cardlist-widget.h"
#include "dialog.h"
#include "gnomecard.h"
#include "gnomecard-prefs.h"
#include "gnomecard-ui.h"
#include "misc.h"
#include "my.h"
#include "sort.h"

#define NAME_COL_WIDTH 100
#define ORG_COL_WIDTH 100
#define PHONE_COL_WIDTH 100
#define EMAIL_COL_WIDTH 100

#define NONE  0
#define FNAME 1

#define IDENT 0
#define GEO 1
#define ORG 2
#define EXP 3

static GMainLoop *main_loop = NULL;

GnomeApp *gnomecard_window;

GtkWidget *cardlist_scrollwin = NULL;
GtkWidget *hpaned = NULL;

gint     gnomecard_selected_row=0;
GtkWidget *tb_next, *tb_prev, *tb_first, *tb_last;
GtkWidget *tb_edit, *tb_find, *tb_save, *tb_del;
GtkWidget *menu_edit, *menu_find, *menu_save, *menu_del;

CardListWidget *gnomecard_cardlist = NULL;

gchar *gnomecard_fname = NULL;
gchar *gnomecard_find_str;
gboolean gnomecard_find_sens;

gboolean gnomecard_changed;


gboolean gnomecard_save(void);

/*
 * Callbacks
 */

static void
gnomecard_about_call(GtkWidget *widget, gpointer data)
{
    dialog_about();
}


static void
gnomecard_edit_card_call(GtkWidget *widget, gpointer data)
{
	GtkTreeModel     *model;
	GtkTreeSelection *selection;
	GtkTreeIter       iter;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gnomecard_cardlist));

	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
		MIMEDirVCard *card;

		gtk_tree_model_get (model, &iter, 0, &card, -1);

		g_assert (MIMEDIR_IS_VCARD (card));

		dialog_edit_card (card);
	}
}


static void
gnomecard_save_call(GtkWidget *widget, gpointer data)
{
    gnomecard_save();
}

static void
gnomecard_save_as_call(GtkWidget *widget, gpointer data)
{
    dialog_save_file();
}

static void
gnomecard_find_card_call(GtkWidget *w, gpointer data)
{
    dialog_find_card();
}

static void
gnomecard_append_call(GtkWidget *w, gpointer data)
{
    dialog_append();
}

static void
gnomecard_setup_call (GtkWidget *w, gpointer data)
{
	GtkWidget *dialog;

	dialog = gncard_prefs_new (NULL);
	gtk_widget_show (dialog);
}


gchar *
gnomecard_join_name (const gchar *pre,
		     const gchar *given,
		     const gchar *add,
		     const gchar *fam,
		     const gchar *suf)
{
    GString *name;

    name = g_string_new (NULL);

    if (pre && *pre) {
	g_string_append (name, pre);
	g_string_append_c (name, ' ');
    }
    if (given && *given) {
	g_string_append (name, given);
	g_string_append_c (name, ' ');
    }
    if (add && *add) {
	g_string_append (name, add);
	g_string_append_c (name, ' ');
    }
    if (fam && *fam) {
	g_string_append (name, fam);
	g_string_append_c (name, ' ');
    }
    if (suf && *suf) {
	g_string_append (name, suf);
	g_string_append_c (name, ' ');
    }
    if (name->len > 0)
	g_string_truncate (name, name->len - 1);
    
    return g_string_free (name, FALSE);
}

void gnomecard_set_app_title (char *title) 
{
	gtk_window_set_title (GTK_WINDOW (gnomecard_window), title);
}

void
gnomecard_set_changed(gboolean val)
{
    gnomecard_changed = val;
    gtk_widget_set_sensitive(tb_save, val);
    gtk_widget_set_sensitive(menu_save, val);
}

static void
gnomecard_set_edit_del(gboolean state)
{
    gtk_widget_set_sensitive(tb_edit, state);
    gtk_widget_set_sensitive(menu_edit, state);
    gtk_widget_set_sensitive(tb_del, state);
    gtk_widget_set_sensitive(menu_del, state);
    gtk_widget_set_sensitive(tb_find, state);
    gtk_widget_set_sensitive(menu_find, state);
}

gboolean
gnomecard_save(void)
{
	GError *error = NULL;
	char *bak_fname, *bak_fname2, *cmd, *real_fname;
	struct stat s;
	char i;
	
	if (!gnomecard_fname || gnomecard_fname[0] == '\0')
		real_fname = misc_tilde_expand ("~/.gnome/GnomeCard.gcrd");
	else
		real_fname = g_strdup (gnomecard_fname);
	
	bak_fname = g_malloc (strlen (real_fname) + 3);
	
	for (i = '0'; i <= '9'; i++) {
		
		bak_fname2 = g_strdup (bak_fname);
		sprintf (bak_fname, "%s~%c", real_fname, i);
		
		if (stat (bak_fname, &s) == -1 && errno == ENOENT) {
			g_free (bak_fname2);
			break;
		}
		
		if (i != '0') {
			cmd = g_malloc (strlen (bak_fname) + 
					strlen (bak_fname2) + 8);
		
			sprintf (cmd, "mv -f %s %s", bak_fname, bak_fname2);
			
			system (cmd);
			g_free (cmd);
			g_free (bak_fname2);
		}
	}
	
	cmd = g_malloc (strlen (bak_fname) + strlen (real_fname) + 8);
	sprintf (cmd, "cp -f %s %s", real_fname, bak_fname);
	system (cmd);
	
	g_free (cmd);
	g_free (bak_fname);

	cardlist_widget_save (gnomecard_cardlist, real_fname, &error);

	g_free (real_fname);

	if (error) {
		gncard_ui_show_error (NULL, error);
		return FALSE;
	}

	gnomecard_set_changed (FALSE);

	return TRUE;
}


/* Returns TRUE if the cards were destroyed. FALSE if canceled */
gint
gnomecard_destroy_cards (void)
{
	if (gnomecard_changed) {
		GtkWidget *w;
		gchar *msg;

		/* FIXME: this requester is nonsense if we are trying to
		 * load a new file while the old one is still being edited.
		 */
		if (!gnomecard_fname || gnomecard_fname[0] == '\0')
			msg = g_strdup (_("The default file has changed. Save?"));
		else
			msg = g_strdup_printf (_("%s has changed. Save?"),
					       gnomecard_fname);
	
		w = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
					   GTK_MESSAGE_WARNING,
					   GTK_BUTTONS_NONE,
					   msg);

		gtk_dialog_add_buttons (GTK_DIALOG (w),
					_("Don't Save"), 1,
					_("Don't Quit"), GTK_RESPONSE_CANCEL,
					_("Save and Quit"), 2,
					NULL);

		gtk_widget_show(w);
	
		switch (gtk_dialog_run (GTK_DIALOG (w))) {
		case GTK_RESPONSE_NONE:
		case GTK_RESPONSE_CANCEL:
			gtk_widget_destroy (w);
			return FALSE;
		case 1:
			break;
		case 2:
			gnomecard_save ();
			break;
		}

		gtk_widget_destroy (w);
	}

	cardlist_widget_clear (gnomecard_cardlist);

	gnomecard_set_changed (FALSE);
    
	return TRUE;
}

static void gnomecard_new_card (GtkWidget *widget, gpointer data)
{
	MIMEDirVCard *card;

	card = mimedir_vcard_new ();
	cardlist_widget_append_card (gnomecard_cardlist, card);

	dialog_edit_card (card);

	gnomecard_set_changed (TRUE);
}

void gnomecard_quit(GtkWidget *widget, gpointer data)
{
	if (gnomecard_destroy_cards ()) {
		gint x, y, pos;
	
		x = GTK_WIDGET (gnomecard_window)->allocation.width;
		y = GTK_WIDGET (gnomecard_window)->allocation.height;
		pos = GTK_PANED(hpaned)->child1_size;
		gnome_config_set_int("/GnomeCard/CardDisplay/width", x);
		gnome_config_set_int("/GnomeCard/CardDisplay/height", y);
		gnome_config_sync ();

		gtk_widget_destroy(GTK_WIDGET (gnomecard_window));

		g_main_loop_quit (main_loop);
	}
}

static void
gnomecard_delete_current_card(GtkWidget *widget, gpointer data)
{
	MIMEDirVCard *vcard;

	vcard = cardlist_widget_get_current_card (gnomecard_cardlist);
	if (vcard) {
		cardlist_widget_remove_card (gnomecard_cardlist, vcard);
	
		gnomecard_set_changed (TRUE);
	}
}

gint gnomecard_delete(GtkWidget *w, GdkEvent *e, gpointer data)
{
	if (gnomecard_destroy_cards()) {
		gtk_widget_destroy(GTK_WIDGET (gnomecard_window));

		g_main_loop_quit (main_loop);
	}
	  
	return TRUE;
}

void gnomecard_spawn_new(GtkWidget *widget, gpointer data)
{
	GtkWidget *w;
	int pid;
	char *text[] = { "gnomecard", NULL };
	
	pid = fork();
	if (pid == 0) { /* child */
		if (execvp("gnomecard", text) == -1) {
			w = gtk_message_dialog_new(NULL, 0,
						   GTK_MESSAGE_ERROR,
						   GTK_BUTTONS_OK,
						   _("A new GnomeCard could not be spawned. Maybe it is not in your path."));
			gtk_widget_show(w);
		}
		exit (1);
	}
}

GnomeUIInfo filemenu[] = {

	GNOMEUIINFO_MENU_NEW_ITEM(N_("New"), N_("Create a new card file"),
				  gnomecard_destroy_cards, NULL),

	GNOMEUIINFO_MENU_OPEN_ITEM(dialog_open, NULL),

	{GNOME_APP_UI_ITEM, N_("_Insert..."),
	N_("Add the contents of another card file"),
	gnomecard_append_call, NULL, NULL,
	GNOME_APP_PIXMAP_STOCK, GTK_STOCK_OPEN, 0, 0, NULL},

	{GNOME_APP_UI_ITEM, N_("Open _Default"),
	 N_("Open the default file"),
	 dialog_open_default, NULL, NULL,
	 GNOME_APP_PIXMAP_STOCK, GTK_STOCK_OPEN, 0, 0, NULL},
	
	GNOMEUIINFO_MENU_SAVE_ITEM(gnomecard_save_call, NULL),

	GNOMEUIINFO_MENU_SAVE_AS_ITEM(gnomecard_save_as_call, NULL),

	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_MENU_EXIT_ITEM(gnomecard_quit, NULL),

	GNOMEUIINFO_END
};

GnomeUIInfo sortradios[] = {
	
	{GNOME_APP_UI_ITEM, N_("By Card Name"), N_("Sort by card name"),
	 gnomecard_sort_cards, NULL, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
	
	{GNOME_APP_UI_ITEM, N_("By Name"), N_("Sort by real name"),
	 gnomecard_sort_cards, NULL, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
	
	{GNOME_APP_UI_ITEM, N_("By Last Name"), N_("Sort by last name"),
	 gnomecard_sort_cards, NULL, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
	
	{GNOME_APP_UI_ITEM, N_("By E-mail"), N_("Sort by e-mail address"),
	 gnomecard_sort_cards, NULL, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},

	{GNOME_APP_UI_ITEM, N_("By Organization"), N_("Sort by organization"),
	 gnomecard_sort_cards, NULL, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
	
	{GNOME_APP_UI_ENDOFINFO}
};

GnomeUIInfo sortmenu[] = {
	
	GNOMEUIINFO_RADIOLIST(sortradios),

	{GNOME_APP_UI_ENDOFINFO}
};

GnomeUIInfo editmenu[] = {
	{GNOME_APP_UI_ITEM, N_("_New card"), N_("Create new card"),
	 gnomecard_new_card, NULL, NULL,
	GNOME_APP_PIXMAP_STOCK, GTK_STOCK_NEW, 0, 0, NULL},

	GNOMEUIINFO_MENU_PROPERTIES_ITEM (gnomecard_edit_card_call, NULL),

	{GNOME_APP_UI_ITEM, N_("_Delete card"), N_("Remove card"),
	 gnomecard_delete_current_card, NULL, NULL,
	GNOME_APP_PIXMAP_STOCK, GTK_STOCK_DELETE, 0, 0, NULL},
	
	{GNOME_APP_UI_SEPARATOR},

	GNOMEUIINFO_MENU_FIND_ITEM (gnomecard_find_card_call, NULL),

	{GNOME_APP_UI_SUBTREE, N_("Sort"), N_("Set sort criteria"),
	 sortmenu, NULL, NULL,
	 GNOME_APP_PIXMAP_STOCK, GTK_STOCK_SORT_ASCENDING, 0, 0, NULL},
	
	{GNOME_APP_UI_ENDOFINFO}
};

GnomeUIInfo settingsmenu[] = {
		GNOMEUIINFO_MENU_PREFERENCES_ITEM(gnomecard_setup_call, NULL),
		GNOMEUIINFO_END
};

GnomeUIInfo helpmenu[] = {
		GNOMEUIINFO_HELP("gnomecard"),
		GNOMEUIINFO_MENU_ABOUT_ITEM(gnomecard_about_call, NULL),
		GNOMEUIINFO_END
};

GnomeUIInfo mainmenu[] = {
        GNOMEUIINFO_MENU_FILE_TREE(filemenu),

	GNOMEUIINFO_MENU_EDIT_TREE(editmenu),

	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),

	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),

	GNOMEUIINFO_END
};

void gnomecard_sort_by_fname(GtkWidget *w, gpointer data)
{
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (sortradios[0].widget), TRUE);
}


static void
gnomecard_list_selection_changed (GtkTreeSelection *selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter   iter;

	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
		MIMEDirVCard *card;

		/* Widget sensibility */

		gnomecard_set_edit_del (TRUE);

		/* Update card display */

		gtk_tree_model_get (model, &iter, 0, &card, -1);
		g_assert (card != NULL && MIMEDIR_IS_VCARD (card));
		gnomecard_update_canvas (NULL, card);
	} else {
		/* Widget sensibility */

		gnomecard_set_edit_del (FALSE);

		/* Update card display */

		gnomecard_clear_canvas (NULL);
	}
}


static void
gnomecard_list_row_activated (GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
{
	GtkTreeModel     *model;
	GtkTreeIter       iter;
	MIMEDirVCard     *card;

	g_return_if_fail (tv != NULL);
	g_return_if_fail (GTK_IS_TREE_VIEW (tv));

	model = gtk_tree_view_get_model (tv);

	gtk_tree_model_get_iter (model, &iter, path);

	gtk_tree_model_get (model, &iter, 0, &card, -1);

	g_assert (MIMEDIR_IS_VCARD (card));

	dialog_edit_card (card);
}


static GtkWidget *gnomecard_create_toolbar(void)
{
	GtkWidget *toolbar;

	toolbar = gtk_toolbar_new();

	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
				 GTK_STOCK_OPEN,
				 _("Open a file"), "",
				 GTK_SIGNAL_FUNC(dialog_open), NULL,
				 -1);

	tb_save = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
					   GTK_STOCK_SAVE,
					   _("Save changes"), "",
					   GTK_SIGNAL_FUNC(gnomecard_save_call), NULL,
					   -1);

	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
				 GTK_STOCK_NEW,
				 _("Create a new card"), "",
				 GTK_SIGNAL_FUNC(gnomecard_new_card), NULL, -1);

	tb_edit = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
					   GTK_STOCK_PROPERTIES,
					   _("Edit card"), "",
					   GTK_SIGNAL_FUNC(gnomecard_edit_card_call), NULL,
					   -1);

	tb_del = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
					  GTK_STOCK_DELETE,
					  _("Delete card"), "",
					  GTK_SIGNAL_FUNC(gnomecard_delete_current_card), NULL,
					  -1);

	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

	tb_find = gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),
					   GTK_STOCK_FIND,
					   _("Search a card"), "",
					   GTK_SIGNAL_FUNC(gnomecard_find_card_call), NULL,
					   -1);

	return toolbar;
}


void gnomecard_init(void)
{
	GtkWidget *widget, *scrollwin, *list;
        GnomeAppBar *appbar;
	GtkWidget *canvas;
	GtkTreeSelection *selection;
	gint x, y, pos;

	gnomecard_window = GNOME_APP(gnome_app_new("GnomeCard", 
						   "GnomeCard: Default"));
	x = gnome_config_get_int("/GnomeCard/CardDisplay/width=0");
	y = gnome_config_get_int("/GnomeCard/CardDisplay/height=0");
	gtk_widget_set_size_request (GTK_WIDGET (gnomecard_window), x, y);

	appbar = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE,
					       GNOME_PREFERENCES_USER));
	gnome_app_set_statusbar(gnomecard_window, GTK_WIDGET(appbar));

	gtk_window_set_wmclass(GTK_WINDOW(gnomecard_window), "GnomeCard",
			       "GnomeCard");

	g_signal_connect (G_OBJECT (gnomecard_window), "delete_event",
			  G_CALLBACK (gnomecard_delete), NULL);
	
	hpaned = gtk_hpaned_new();
	gtk_container_set_border_width(GTK_CONTAINER(hpaned), GNOME_PAD_SMALL);
	gtk_widget_show(hpaned);
	
	gnome_app_create_menus(GNOME_APP(gnomecard_window), mainmenu);

	gnome_app_install_menu_hints(GNOME_APP(gnomecard_window), mainmenu);
	widget = gnomecard_create_toolbar();
	gnome_app_set_toolbar(GNOME_APP(gnomecard_window), GTK_TOOLBAR(widget));

	gnome_app_set_contents(GNOME_APP(gnomecard_window), hpaned);

	cardlist_scrollwin = scrollwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwin), GTK_SHADOW_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrollwin), GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);

	list = cardlist_widget_new ();
	gnomecard_cardlist = CARDLIST_WIDGET (list);
	gtk_widget_show (list);
	gtk_container_add (GTK_CONTAINER (scrollwin), list);
	gtk_widget_show(scrollwin);
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
	g_signal_connect (G_OBJECT (selection), "changed",
			  G_CALLBACK (gnomecard_list_selection_changed), NULL);
	g_signal_connect (G_OBJECT (list), "row-activated",
			  G_CALLBACK (gnomecard_list_row_activated), NULL);

	gtk_paned_pack1(GTK_PANED(hpaned), cardlist_scrollwin, TRUE, TRUE);

	canvas = gnomecard_canvas_new();
	scrollwin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
				   GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
	gtk_container_add(GTK_CONTAINER(scrollwin), canvas);
	gtk_widget_show(canvas);
	gtk_widget_show(scrollwin);
	gtk_paned_pack2(GTK_PANED(hpaned), scrollwin, TRUE, TRUE);
	pos = gnome_config_get_int("/GnomeCard/CardDisplay/position=250");
	gtk_paned_set_position(GTK_PANED(hpaned), pos);

	/* FIXME ? - ugly mechanism to be able to enable/disable buttons on */
	/*           toolbar                                                */
	menu_save = filemenu[3].widget;
	menu_edit = editmenu[1].widget;
	menu_del  = editmenu[2].widget;
	menu_find = editmenu[4].widget;

	gnomecard_find_sens = gnome_config_get_bool("/GnomeCard/find/sens=False");
	gnomecard_find_str = gnome_config_get_string("/GnomeCard/find/str=");

	gnomecard_set_changed(FALSE);

	if (gnomecard_fname) {
		GError *error = NULL;

		if (!cardlist_widget_open (gnomecard_cardlist, gnomecard_fname, &error))
			gncard_ui_show_error (NULL, error);
	}

	gtk_widget_set_size_request (GTK_WIDGET (gnomecard_window), 600, 400);

	gtk_widget_show(GTK_WIDGET (gnomecard_window));
}

int main (int argc, char *argv[])
{
	/* I18n setup */

	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	gnome_program_init ("gnomecard", VERSION,
			    LIBGNOMEUI_MODULE,
			    argc, argv,
			    GNOME_PROGRAM_STANDARD_PROPERTIES,
			    NULL);

	gncard_ui_init ();

	gnomecard_init();

	main_loop = g_main_loop_new (NULL, FALSE);
	g_main_loop_run (main_loop);
	g_main_loop_unref (main_loop);

	return 0;
}
